/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.ajax;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.mentawai.core.Action;
import org.mentawai.core.Consequence;
import org.mentawai.core.ConsequenceException;

/**
 * @author Rubem Azenha (rubem.azenha@gmail.com)
 *         <p>
 *         A Consequence for using Asynchronous JavaScript And XML, also know as
 *         Ajax. This consequence must receive an AjaxRender in its constructor.
 *         The AjaxRender is responsible for getting the action result and
 *         renderize it into a AjaxResponse implementation.
 *         </p>
 *         <p>
 *         Each implementation of the AjaxRender can expect diferents values in
 *         the action`s output and generates differents text results. The result
 *         could be a XML structure, a JSON object, a HTML or even a simple
 *         text.
 *         </p>
 *         <p>
 *         This consequence gets the rendered String and write it`s content to
 *         the client.
 *         </p>
 *         <p>
 *         Then you can use your favorite JavaScript Ajax library (or use your
 *         own library) to parse the generated result and interact it in your
 *         code.
 *         </p>
 */
public class AjaxConsequence implements Consequence {

    private final AjaxRender ajaxRender;

    /**
     * @param render
     *            The render used to renderize the action.
     */
    public AjaxConsequence(AjaxRender render) {
        ajaxRender = render;
    }

    /**
     * Prints in the request's output the ajax response. The ajax response is
     * supplied by the AjaxRender.
     */
    public void execute(Action a, HttpServletRequest req,
            HttpServletResponse res) throws ConsequenceException {

        try {
            res.setContentType(ajaxRender.getContentType());
            String s = this.ajaxRender.renderize(a);
            
            res.setHeader("Cache-Control","no-cache"); // HTTP 1.1
            res.setHeader("Pragma","no-cache"); // HTTP 1.0
            res.setDateHeader ("Expires", 0); // prevents caching at the proxy
                                                // server
            
            PrintWriter printWriter = res.getWriter();
            printWriter.print(s);
        } catch (IOException e) {
            throw new RuntimeException(
                    "Exception while writing the renderized document: "
                            + e.getMessage(), e);
        } catch (Exception e) {
            throw new RuntimeException(
                    "Exception while renderizing with render "
                            + ajaxRender.getClass() + ": " + e.getMessage(), e);
        }

    }
}