package org.mentawai.core;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.mentawai.util.InjectionUtils;
import org.mentawai.util.StringUtils;

public abstract class AbstractInput implements Input {

	/**
     * Calls getStringValue() and tries to convert the string to boolean.
     */
    public boolean getBooleanValue(String name) {
        
        String value = getStringValue(name);

        if (value == null || value.trim().equals("")) {
            // throw new NullPointerException(name + " is not defined");
            return false;
        }
        
        if (value.equalsIgnoreCase("false")) return false;
        
        if (value.equalsIgnoreCase("true")) return true;
        
        int x = -1;
        
        try {
            x = Integer.parseInt(value);
            
            if (x == 0) return false;
            else if (x == 1) return true;
            
        } catch(Exception e) { }
        
        throw new InputException("Could not convert input to boolean: " + name + " (" + value + ")");
    }
	
	public boolean getBooleanValue(String name, boolean def) {
        
        String value = getStringValue(name);

        if (value == null || value.trim().equals("")) {
            // throw new NullPointerException(name + " is not defined");
            return def;
        }
        
        return getBooleanValue(name);
        
    }
	
	public Date getDate(String name) {
		return getDate(name, null, DateFormat.DEFAULT);
	}
	
	public Date getDate(String name, int style) {
		return getDate(name, null, style);
	}

	public Date getDate(String name, String pattern) {
		return getDate(name, pattern, -1);
	}
	
	private Date getDate(String name, String pattern, int style) {
		
		// first check whether we already have an Date...
		
		Object obj = getValue(name);
		
		if (obj instanceof Date) {
			
			return (Date) obj;
		}
		
		// if not parse...
		
		String date = getStringValue(name);
		
		if(StringUtils.isEmpty(date)){
			return null;
		}
		
		try {
			
			DateFormat df;
			if(pattern != null){
				df = new SimpleDateFormat(pattern);
			} else if(style != -1){
				df = DateFormat.getDateInstance(style, getLocale());
			} else {
				throw new IllegalStateException("Should never be here!");
			}
			return df.parse(date);
			
		} catch(ParseException e) {
			
            throw new InputException("Could not convert input to date: " + name + " (" + date + ")");
		}
	}
    
    
    /**
     * Calls getStringValue() and tries to convert the string to a double.
     */
    public double getDoubleValue(String name) {
        
        String value = getStringValue(name);

        if (value == null || value.trim().equals("")) {
            // throw new NullPointerException(name + " is not defined");
            return -1;
        }

        try {
            return Double.parseDouble(value);
        } catch (NumberFormatException e) {
            throw new InputException("Could not convert input to double: "
                    + name + " (" + value + ")");
        }
    }
    
    public double getDoubleValue(String name, double def) {
        
        String value = getStringValue(name);

        if (value == null || value.trim().equals("")) {
            
            return def;
        }
        
        return getDoubleValue(name);
    }
    
    /**
     * Calls getStringValue() and tries to convert the string to a float.
     */
    public float getFloatValue(String name) {
        
        String value = getStringValue(name);

        if (value == null || value.trim().equals("")) {
            // throw new NullPointerException(name + " is not defined");
            return -1;
        }

        try {
            return Float.parseFloat(value);
        } catch (NumberFormatException e) {
            throw new InputException("Could not convert input to float: "
                    + name + " (" + value + ")");
        }
    }
    
    public float getFloatValue(String name, float def) {
        
        String value = getStringValue(name);

        if (value == null || value.trim().equals("")) {

            return def;
        }
        
        return getFloatValue(name);

    }
    
    public int getIntValue(String name) {
        String value = getStringValue(name);
        if (value == null || value.trim().equals("")) {
            //throw new NullPointerException("No value defined for " + name);
            return -1;
        }
        try {
            return Integer.parseInt(value);
        } catch(Exception e) {
            throw new InputException("Could not convert input to number: " + name + " (" + value + ")");
        }
    }
    
    public int getIntValue(String name, int def) {
        
        String value = getStringValue(name);

        if (value == null || value.trim().equals("")) {
            return def;
        }
        
        return getIntValue(name);

    }
    
    public int[] getIntValues(String name) {
		String[] values = getStringValues(name);

		if (values == null || values.length == 0) {
			return null;
		}

		int [] ret = new int[values.length];

		for(int i = 0; i <values.length; i++) {
			try {
				ret[i] = Integer.parseInt(values[i]);
			}
			catch(NumberFormatException e) {
				throw new InputException("Could not convert input to numbers: " + name);
			}
		}
		return ret;
	}
    
    protected abstract Locale getLocale() ;
    
    /**
     * Calls getStringValue() and tries to convert the string to a long.
     */
    public long getLongValue(String name) {
        String value = getStringValue(name);

        if (value == null || value.trim().equals("")) {
            // throw new NullPointerException(name + " is not defined");
            return -1;
        }

        try {
            return Long.parseLong(value);
        } catch (NumberFormatException e) {
            throw new InputException("Could not convert input to long: "
                    + name + " (" + value + ")");
        }
    }

	public long getLongValue(String name, long def) {
        String value = getStringValue(name);

        if (value == null || value.trim().equals("")) {
            return def;
        }
        
        return getLongValue(name);
    }

	
	public <E> E getObject(Class<? extends E> klass) {

		return getObject(klass, null);

	}

	public <E> E getObject(Class<? extends E> klass, String prefix) {

		return getObject(klass, prefix, true, true);

	}

	public <E> E getObject(Class<E> klass, String prefix, boolean tryField,
			boolean tryToConvert) {

		try {

			E target = klass.newInstance();

			InjectionUtils.getObject(target, this, getLocale(), tryField,
					prefix, tryToConvert);

			return target;

		} catch (Exception e) {

			throw new InputException(e);
		}
	}

}
