/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.core;

import java.lang.reflect.Method;
import java.util.LinkedList;
import java.util.List;

import org.mentawai.util.DebugServletFilter;

/**
 * When an action is executed, a chain of filters is created.
 * The last step of any InvocationChain is the action.
 * An action may have one or more filters and global filters.
 * 
 * @author Sergio Oliveira
 */
public class InvocationChain {

	private LinkedList<Filter> filters = new LinkedList<Filter>();
	private Action action;
    private String innerAction = null;
    private final String actionName;

	/**
	 * Creates an InvocationChain for this action.
	 * 
	 * @param action The action for what this InvocationChain will be created.
	 */
	public InvocationChain(String actionName, Action action) {
		
		this.actionName = actionName;
		
		this.action = action;
	}

	/**
	 * Gets the action of this InvocationChain
	 * 
	 * @return The action of this InvocationChain
	 */
	public Action getAction() {
		return action;
	}

	void addFilter(Filter filter) {
		filters.add(filter);
	}

	void addFilters(List<Filter> list) {
		filters.addAll(list);
	}
    
	/**
	 * Invoke and execute the next step in this InvocationChain.
	 * This can be the next filter or the action.
	 * 
	 * @return The result of a filter or the action.
	 */
	public String invoke() throws Exception {
        
        if (Controller.debugMode) DebugServletFilter.debugInputOutput(action);
            
		if (!filters.isEmpty()) {
			Filter f = (Filter) filters.removeFirst();
            
            if (Controller.debugMode) {
                
                StringBuffer sb = DebugServletFilter.getDebug(action);
                
                DebugServletFilter.debug(sb, f);
                
            }
            
			return f.filter(this);
		}
        
        if (Controller.debugMode) {
            
            StringBuffer sb = DebugServletFilter.getDebug(action);
            
            DebugServletFilter.debug(sb, innerAction, false);
            
        }
        
        String result = null;
        
		if (innerAction != null) {
			Method m = getMethod(innerAction);
			if (m != null) {
				try {
					result = (String) m.invoke(action, new Object[0]);
				} catch(Exception e) {
					throw new ActionException(e);
				}
			} else {
				throw new ActionException("The inner action does not exist: " + innerAction);
			}
		} else {
			result = action.execute();
		}
        
        return result;
	}
	
	private Method getMethod(String innerAction) {
		try {
			Method m = action.getClass().getMethod(innerAction, new Class[0]);
			if (m != null) {
				return m;
			}			
		} catch(Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * Sets an inner action to be executed.
	 * An inner action is a method inside the action implementation that can be executed instead of the execute() method.
	 *
	 * @param innerAction The name of the method to be executed as an inner action
	 */
	public void setInnerAction(String innerAction) {
		this.innerAction = innerAction;
	}
    
    /**
     * Returns the inner action being executed in the invocation chain.
     *
     * @return The innner action or null if there is no inner action being executed.
     * @since 1.2.1
     */
    public String getInnerAction() {
        return innerAction;
    }

    /**
     * Returns the name of the action being executed in the invocation chain.
     * 
     * @return The action name like HelloMentawai
     * @since 1.8
     */
    public String getActionName() {
    	return actionName;
    }
    
    /**
     * Returns the filters of this invocation chain.
     * 
     * @return all filters of this invocation chain
     * @since 1.4
     */
    public List<Filter> getFilters() {
        
        return filters;
        
    }
        
}