/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.filter;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.mentawai.converter.ConversionException;
import org.mentawai.converter.Converter;
import org.mentawai.core.Action;
import org.mentawai.core.Filter;
import org.mentawai.core.FilterException;
import org.mentawai.core.Input;
import org.mentawai.core.InvocationChain;

/**
 * A filter to perform conversions of the action input values.
 * Use this filter if you want to convert values of your action input.
 * 
 * By default, after the action is executed, this filter will restore the old values it converted.
 * 
 * You can disable that through the <i>setRestore</i> method.
 * 
 * @author Sergio Oliveira
 */
public abstract class ConversionFilter implements Filter {
	
	private Map<String, Converter> converters = new HashMap<String, Converter>();
	
	private boolean isLoaded = false;
	
	private boolean restore = true;
	
    /**
     * Creates a ConversionFilter.
     */
	public ConversionFilter() {
	}
	
	/**
	 * Creates a ConversionFilter.
	 * 
	 * @param restore Restore the converted values after the action execution?
	 * @since 1.2
	 */
	public ConversionFilter(boolean restore) {
		this.restore = restore;
	}
	
	/*
	 * Lazy initialization of converters! 
	 */
	private Map<String, Converter> getConverters() {
		
		if (isLoaded) return converters;
		
		synchronized(this) {
			
			if (!isLoaded) {
			
				initConverters();
			
				isLoaded = true;
				
			}
		}
		
		return converters;
	}
	
	/**
	 * Should this filter restore the old values after the action has executed?
	 * 
	 * @param restore true to restore the values.
	 * @since 1.2
	 */
	public void setRestore(boolean restore) {
		this.restore = restore;
	}
	
    /**
     * Override this abstract method to add converters for the fields you want to convert.
     */
	public abstract void initConverters();
	
    /**
     * Adds a converter for the given field.
     *
     * @param field The field to convert.
     * @param converter The converter to use.
     * @return This ConversionFilter object (<i>this</i>) so you can chain methods
     */
	public ConversionFilter add(String field, Converter converter) {
		converters.put(field, converter);
		return this;
	}
	
	public String filter(InvocationChain chain) throws Exception {
		Action action = chain.getAction();
		Input input = action.getInput();
		
		Map<String, Object> initialValues = null;
		
		if (restore) {
			initialValues = new HashMap<String, Object>();
		}
		
		Map<String, Converter> converters = getConverters();
		
		try {
			Iterator<String> iter = converters.keySet().iterator();
			while(iter.hasNext()) {
				String field = iter.next();
				Converter c = converters.get(field);
				
				Object oldValue = input.getValue(field);
				
				Object converted = c.convert(field, action);
				
				input.setValue(field, converted);
				
				if (restore) initialValues.put(field, oldValue);
			}
		} catch(ConversionException e) {
			e.printStackTrace();
			throw new FilterException(e);
		}
		
		try {
			return chain.invoke();
		} catch(Exception e) {
			throw e;
		} finally {
			if (restore) {
				
				Iterator<String> iter = initialValues.keySet().iterator();
				
				while(iter.hasNext()) {
					
					String field = iter.next();
					
					Object value = initialValues.get(field);
					
					input.setValue(field, value);
					
				}
			}
		}
	}
    
    public void destroy() { }
}
		