/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.filter;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
import java.util.WeakHashMap;

import org.mentawai.core.Action;
import org.mentawai.core.Filter;
import org.mentawai.core.InputWrapper;
import org.mentawai.core.InvocationChain;
import org.mentawai.log.Debug;
import org.mentawai.util.InjectionUtils;

/**
 * A filter that will do AUTO-WIRING of dependencies in a totaly transparent way.
 * 
 * @author Sergio Oliveira
 */
public class DIFilter extends InputWrapper implements Filter {
	
	private static final String NAME = "DIFilter";

    private Map<String, AccessibleObject> cache = new HashMap<String, AccessibleObject>();

    private Map received = new WeakHashMap();

    private boolean tryField = true;

    private final String sourceKey;

	private final String attrName;

    private final Class sourceClass;

    /**
     * Create a dependency of type sourceClass and name sourceKey, in other words,
     * all objects fetched from the action input will be checked for that dependency.
     * 
     * If they have a setter method or an instance variable for the property <i>sourceKey</i>
     * of the type <i>sourceClass</i>, the filter will look for this property in the action input
     * using <i>sourceKey</i> and inject.
     * 
     *  The filter also validates whether the dependency it finds in the action input through <i>sourceKey</i>
     *  is of the type <i>sourceClass</i>. If it is not it will not inject.
     *  
     *  The pseudo-code for this process is:
     *  
     *  - An object TARGET is requested from the action input.
     *  
     *  Object TARGET = input.get(anythiner here);
     *  
     *  - The DIFilter checks the TARGET object to see if it has a dependency of type <i>sourceClass</i> with 
     *  the property name <i>sourceKey</i>. (Ex: setConn(Connection c) or Connection conn)
     *  
     *  - If the dependency is found, it will look for a compatible object inside the action input using <i>sourceKey</i> as the key.
     *  
     *  Object SOURCE = input.get(sourceKey); // look for the dependency to inject in our TARGET object
     *  
     *  - If SOURCE is found and SOURCE is of a type compatible with <i>sourceClass</i> it will be injected in the TARGET object.
     *  
     * @param sourceKey The name of the attribute the target class must have and also the key that will be used to locate the dependency inside the action input.
     * @param sourceClass The type of the dependency.
     */
    public DIFilter(String sourceKey, Class sourceClass) {

        this(sourceKey, sourceKey, sourceClass);

    }

    /**
     * Create a dependency of type sourceClass and name attrName, in other words,
     * all objects fetched from the action input will be checked for that dependency.
     * 
     * If they have a setter method or an instance variable for the property <i>atrrName</i>
     * of the type <i>sourceClass</i>, the filter will look for this property in the action input
     * using <i>sourceKey</i> and inject.
     * 
     *  The filter also validates whether the dependency it finds in the action input through <i>sourceKey</i>
     *  is of the type <i>sourceClass</i>. If it is not it will not inject.
     *  
     *  The pseudo-code for this process is:
     *  
     *  - An object TARGET is requested from the action input.
     *  
     *  - The DIFilter checks the TARGET object to see if it has a dependency of type <i>sourceClass</i> with 
     *  the property name <i>attrName</i>. (Ex: setConn(Connection c) or Connection conn)
     *  
     *  - If the dependency is found, it will look for an object inside the action input to inject in TARGET. It will use the <i>sourceKey</i> as the key.
     *  
     *  Object SOURCE = input.get(sourceKey); // look for the dependency to inject in our TARGET object
     *  
     *  - If SOURCE is found and SOURCE is of a type compatible with <i>sourceClass</i> it will be injected in the TARGET object.
     *
     * @param sourceKey The key used to look for the source object (the object that TARGET depends on) inside the action input.
     * @param attrName The name of the attribute that the target class must have in order to receive the dependency.
     * @param sourceClass The type of the dependency.
     */
    public DIFilter(String sourceKey, String attrName, Class sourceClass) {

        this(sourceKey, attrName, sourceClass, true);

    }

    /**
     * Same as the other constructors, but here you can say whether you want to inject in a field or not.
     * 
     * The default is TRUE, in other words, it will try to inject in a field through reflection, even if this field is private.
     * 
     * If you pass FALSE to tryField it will only try to inject through setters!
     * 
     * @param sourceKey
     * @param attrName
     * @param sourceClass
     * @param tryField
     */
    public DIFilter(String sourceKey, String attrName, Class sourceClass, boolean tryField) {

        this.sourceKey = sourceKey;

        this.attrName = attrName;

        this.sourceClass = sourceClass;

        this.tryField = tryField;

    }

    public String toString() {

        StringBuffer sb = new StringBuffer(128);

        sb.append("DIFilter: sourceKey=").append(sourceKey).append(" attrName=").append(attrName).append(" sourceClass=").append(sourceClass.getName());
        sb.append(" tryField=").append(tryField);

        return sb.toString();
    }

    public void setTryField(boolean tryField) {
        this.tryField = tryField;
    }

	public String filter(InvocationChain chain) throws Exception {

		Action a = chain.getAction();

		setInput(a.getInput());

		a.setInput(this);

		return chain.invoke();

	}

    /*
     * Check if the target class has the dependency. Cache for performance!
     */
    private Object getMethodOrField(Class<? extends Object> target) {

        String className = target.getName();

        // first check cache...

        Object obj = null;

        synchronized(cache) {

            obj = cache.get(className);

        }

        if (obj != null) return obj;

        Method m = InjectionUtils.findMethodToInject(target, attrName, sourceClass);
        
        if (Debug.ENABLED) Debug.log(NAME, "Finding method:", "target=", target.getName(), "attrName=", attrName, "sourceClass=", sourceClass.getName(), "methodFound=", m == null ? "NULL" : m.getName());

        if (m != null) {

            synchronized(cache) {

                cache.put(className, m);
            }

            return m;

        }
        
        if (tryField) {

            Field f = InjectionUtils.findFieldToInject(target, attrName, sourceClass);
            
            if (Debug.ENABLED) Debug.log(NAME, "Finding field:", "target=", target.getName(), "attrName=", attrName, "sourceClass=", sourceClass.getName(), "methodFound=", f == null ? "NULL" : f.getName());
    
            if (f != null) {
    
                synchronized(cache) {
    
                    cache.put(className, f);
    
                }
    
                return f;
            }
        }

        synchronized(cache) {

            cache.put(className, null);

        }

        return null;

    }

    /*
     * Check if this object has already received the dependency, because we don't want to
     * inject every time. Injecting just once is the correct behaviour.
     */
    private boolean hasAlreadyReceived(Object target) {

        synchronized(received) {

            return received.containsKey(target);

        }
    }

    /*
     * Flag that this object has already received the dependency.
     */
    private void setAlreadyReceived(Object target) {

        synchronized(received) {

            received.put(target, null);

        }
    }


	public void setValue(String key, Object value) {

	    super.setValue(key, value);

	    //"toutch" the object to inject the dependencies
	    getValue(key);

	}


	public Object getValue(String key) {

        Object target = super.getValue(key);

        if (target != null) {

            // has dependency ?
            Object obj = getMethodOrField(target.getClass());
            
            if (Debug.ENABLED) Debug.log(NAME, "Has dependency?", "target =", target.getClass().getName(), "key =", key, "depFound =", obj); 

            if (obj != null) {
            	
            	boolean hasAlreadyReceived = hasAlreadyReceived(target);
            	
            	if (Debug.ENABLED) Debug.log(NAME, "HasAlreadyReceived?", "target =", target.getClass().getName(), "hasAlreadyReceived =", hasAlreadyReceived);

                // has already received the dependency?
                if (!hasAlreadyReceived) {

                    // get dependency from action input
                    Object source = super.getValue(sourceKey);
                    
                    boolean isAssignable = source != null && sourceClass.isAssignableFrom(source.getClass());
                    
                    if (Debug.ENABLED) Debug.log(NAME, "isAssignable?", "source =", source == null ? "NULL" : source.getClass().getName(), "isAssignable =", isAssignable, "sourceKey =", sourceKey);

                    // check if we can find the dependency and if it is assignable to the target dependency
                    if (isAssignable) {

                        if (obj instanceof Method) {

                            Method m = (Method) obj;

                            try {
                            	
                            	if (Debug.ENABLED) Debug.log(NAME, "Injecting thru method!", "method =", m.getName(), "target =", target.getClass().getName(), "source =", source);

                                // inject
                                m.invoke(target, new Object[] { source });

                                // flag it has received
                                setAlreadyReceived(target);
                                
                                if (Debug.ENABLED) Debug.log(NAME, "Injecting thru method succeeded!");

                            } catch(Exception e) {
                            	
                            	if (Debug.ENABLED) Debug.log(NAME, "Injecting thru method failed!", "exception =", e.getMessage());

                                throw new RuntimeException("Cannot inject dependency !!!", e);

                            }

                        } else if (obj instanceof Field) {

                            Field f = (Field) obj;

                            try {
                            	
                            	if (Debug.ENABLED) Debug.log(NAME, "Injecting thru field!", "field =", f.getName(), "target =", target.getClass().getName(), "source =", source);

                                // inject in the field !!!!
                                f.set(target, source);

                                // flag it has received
                                setAlreadyReceived(target);
                                
                                if (Debug.ENABLED) Debug.log(NAME, "Injecting thru field succeeded!");

                            } catch(Exception e) {
                            	
                            	if (Debug.ENABLED) Debug.log(NAME, "Injecting thru field failed!", "exception =", e.getMessage());

                                throw new RuntimeException("Cannot inject dependency !!!", e);

                            }
                        }
                    }
                }
            }
        }

        return target; // return target nicely with the dependency injected
	}

	public void destroy() { }


}