/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.filter;

import java.util.Collections;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import org.mentawai.core.Action;
import org.mentawai.core.BaseAction;
import org.mentawai.core.Filter;
import org.mentawai.core.InvocationChain;
import org.mentawai.i18n.LocaleManager;
import org.mentawai.message.ClassMessageContext;
import org.mentawai.message.FileMessageContext;
import org.mentawai.message.MessageContext;
import org.mentawai.validation.DynValidatable;
import org.mentawai.validation.Validatable;
import org.mentawai.validation.Validator;

/**
 * A filter to validate the values of an action input.
 * This filter has the same behaviour of a ValidationFilter except
 * that it will get the validation information from the action, if it implements the Validatable interface.
 *
 * @author Sergio Oliveira
 */
public class ValidatorFilter implements Filter {
    
    private static final String DEFAULT_DIR = "/validation";
	
	private String resultForError = BaseAction.ERROR;
    
	private MessageContext masterMsgContext = null;
	
	private Map<ActionUniqueId, Validator> cache = Collections.synchronizedMap(new HashMap<ActionUniqueId, Validator>());
    
    private Map<ActionUniqueId, MessageContext> msgContexts = new Hashtable<ActionUniqueId, MessageContext>();
    
    private String dir = DEFAULT_DIR;
	
    /**
     * Creates a Validator filter.
     */
	public ValidatorFilter() {
        
        if (LocaleManager.isUseMasterForEverything()) {
            masterMsgContext = new FileMessageContext(LocaleManager.getMaster(), "");
        }
	}
    
    public ValidatorFilter(String resultForError) {
        
        this();
        
        this.resultForError = resultForError;
    }
    
    private MessageContext getMessageContext(Class klass) {
        
        if (masterMsgContext != null) return masterMsgContext;
        
        return new ClassMessageContext(klass, dir);
        
    }
	
    /**
     * By default, the filter returns the <i>BaseAction.ERROR</i> when a validation failure happens.
     * You can change that by calling this method.
     *
     * @param resultForError The result to return in case of a validation failure.
     */
	public void setResultForError(String resultForError) {
		this.resultForError = resultForError;
	}
	
    /**
     * Sets the directory where to look for error messages.
     * You should only call this method if you want to change the default directory,
     * which is <i>/validation</i>.
     * Note that calling this method will force the change of the message context
     * of this filter to ClassMessageContext with the given directory.
     *
     * @param dir The directory where to look for error messages.
     */
	public void setDir(String dir) {
		
        this.dir = dir;
        
        msgContexts.clear();
        
	}
	
	public String filter(InvocationChain chain) throws Exception {
		
		Action action = chain.getAction();
		
		ActionUniqueId actionId = new ActionUniqueId(action.getClass().getName(), chain.getInnerAction());
		
        if (action instanceof Validatable) {
        	
        	Validator validator = cache.get(actionId);
            
        	if (validator == null) {
        		
        		synchronized(this) {
        			
        			if (validator == null) {
        	
			            Validatable v = (Validatable) action;
			            
			            validator = new Validator(new HashMap<String, Object>(), new HashMap<String, Object>());
			            
			            v.initValidator(validator, chain.getInnerAction());
			            
			            cache.put(actionId, validator);
	            
        			}
        		}
        	}
        	
            MessageContext msgContext = msgContexts.get(actionId);
            
            if (msgContext == null) {
                
                msgContext = getMessageContext(action.getClass());
                
                msgContexts.put(actionId, msgContext);
                
            }
            
            boolean isOk = validator.validate(action, msgContext);
            
            if (!isOk) {
                return resultForError;
            }        	
        	
        }
        
        if (action instanceof DynValidatable) {
        	
            DynValidatable v = (DynValidatable) action;
            
            Validator validator = new Validator(new HashMap<String, Object>(), new HashMap<String, Object>());
            
            v.prepareValidator(validator, chain.getInnerAction());
            
            MessageContext msgContext = msgContexts.get(actionId);
            
            if (msgContext == null) {
                
                msgContext = getMessageContext(action.getClass());
                
                msgContexts.put(actionId, msgContext);
                
            }
            
            boolean isOk = validator.validate(action, msgContext);
            
            if (!isOk) {
                return resultForError;
            }            
            
        }
        
		return chain.invoke();
	}
    
    public void destroy() { }
    
    static class ActionUniqueId {
    	
    	String classname;
    	
    	String innerAction = null;
    	
    	public ActionUniqueId(String classname, String innerAction) {
    		this.classname = classname;
    		this.innerAction = innerAction;
    	}
    	
    	public int hashCode() {
    		
    		return 34544 * classname.hashCode() + 194 * (innerAction == null ? 0 : innerAction.hashCode());
    		
    	}
    	
    	public boolean equals(Object obj) {
    		
    		if (obj instanceof ActionUniqueId) {
    			
    			ActionUniqueId a = (ActionUniqueId) obj;
    			
    			if (a.classname.equals(this.classname)) {
    				
    				if (a.innerAction == null && this.innerAction == null) return true;
    				
    				if (a.innerAction != null && this.innerAction != null && a.innerAction.equals(this.innerAction)) return true;
    				
    			}
    			
    		}
    		
    		return false;
    	}
    }
}
		