/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.util;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.mentawai.core.Input;
import org.mentawai.core.Output;

/**
 * 
 * @author Sergio Oliveira
 */
public class InjectionUtils {

	/**
	 * The character used to separate the prefix from the value name when you
	 * are using the getObject method with a prefix. You can change the value of
	 * this prefix if you want to by changing this static variable.
	 * 
	 * Ex: getObject(User.class, "user") will get all values that begin with
	 * "user.".
	 */
	public static char PREFIX_SEPARATOR = '.';

	private static Map<Class<? extends Object>, Map<String, Object>> settersMaps = new HashMap<Class<? extends Object>, Map<String, Object>>();

	private static Map<Class<? extends Object>, Map<String, Object>> fieldsMaps = new HashMap<Class<? extends Object>, Map<String, Object>>();

	public static void prepareForInjection(Class<? extends Object> klass,
			Map<String, Object> setters, Map<String, Object> fields) {

		StringBuffer sb = new StringBuffer(32);

		Method[] methods = klass.getMethods();

		for (int i = 0; i < methods.length; i++) {

			Method m = methods[i];

			String name = m.getName();

			Class[] types = m.getParameterTypes();

			if (name.startsWith("set") && name.length() > 3
					&& types.length == 1) {

				String var = name.substring(3);

				if (var.length() > 1) {

					sb.delete(0, sb.length());

					sb.append(var.substring(0, 1).toLowerCase());

					sb.append(var.substring(1));

					var = sb.toString();

				} else {

					var = var.toLowerCase();
				}

				m.setAccessible(true);

				if (setters.containsKey(var)) {

					Object obj = setters.get(var);

					if (obj instanceof List) {

						List<Method> list = (List<Method>) obj;

						list.add(m);

					} else {

						List<Object> list = new ArrayList<Object>();

						list.add(obj);

						list.add(m);

						setters.put(var, list);

					}

				} else {

					setters.put(var, m);

				}
			}
		}

		if (fields == null)
			return;

		Field[] f = klass.getDeclaredFields();

		for (int i = 0; i < f.length; i++) {

			Field field = f[i];

			field.setAccessible(true);

			String name = field.getName();

			if (setters.containsKey(name))
				continue;

			fields.put(name, field);

		}
	}

	public static boolean checkPrimitives(Class target,
			Class<? extends Object> source) {

		if (target.equals(int.class) && source.equals(Integer.class))
			return true;

		if (target.equals(boolean.class) && source.equals(Boolean.class))
			return true;

		if (target.equals(byte.class) && source.equals(Byte.class))
			return true;

		if (target.equals(short.class) && source.equals(Short.class))
			return true;

		if (target.equals(char.class) && source.equals(Character.class))
			return true;

		if (target.equals(long.class) && source.equals(Long.class))
			return true;

		if (target.equals(float.class) && source.equals(Float.class))
			return true;

		if (target.equals(double.class) && source.equals(Double.class))
			return true;

		return false;

	}

	public static Object tryToConvert(Object source, Class targetType,
			Locale loc) {

		String value = null;

		if (source instanceof String) {

			value = (String) source;

		} else {

			return null;

		}

		Object newValue = null;

		String className = targetType.getName();

		if (className.equals("int") || className.equals("java.lang.Integer")) {
			int x = -1;
			try {
				x = Integer.parseInt(value);
			} catch (Exception e) {
				return null;
			}
			newValue = new Integer(x);
		} else if (className.equals("long")
				|| className.equals("java.lang.Long")) {
			long x = -1;
			try {
				x = Long.parseLong(value);
			} catch (Exception e) {
				return null;
			}
			newValue = new Long(x);
		} else if (className.equals("float")
				|| className.equals("java.lang.Float")) {
			float x = -1;
			try {
				x = Float.parseFloat(value);
			} catch (Exception e) {
				return null;
			}
			newValue = new Float(x);
		} else if (className.equals("double")
				|| className.equals("java.lang.Double")) {
			double x = -1;
			try {
				x = Double.parseDouble(value);
			} catch (Exception e) {
				return null;
			}
			newValue = new Double(x);
		} else if (className.equals("boolean")
				|| className.equals("java.lang.Boolean")) {
			try {
				int x = Integer.parseInt(value);
				if (x == 1) {
					newValue = Boolean.TRUE;
				} else if (x == 0) {
					newValue = Boolean.FALSE;
				} else {
					return null;
				}
			} catch (Exception e) {
				if (value.equalsIgnoreCase("true")) {
					newValue = Boolean.TRUE;
				} else if (value.equalsIgnoreCase("false")) {
					newValue = Boolean.FALSE;
				} else {
					return null;
				}
			}
		} else if (className.equals("java.util.Date") && loc != null) {
			DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, loc); // change
			// this.
			df.setLenient(false);
			try {
				newValue = df.parse(value);
			} catch (Exception e) {
				return null;
			}
		}

		return newValue;

	}

	public static Class getPrimitiveFrom(Object w) {
		if (w instanceof Boolean) {
			return Boolean.TYPE;
		} else if (w instanceof Byte) {
			return Byte.TYPE;
		} else if (w instanceof Short) {
			return Short.TYPE;
		} else if (w instanceof Character) {
			return Character.TYPE;
		} else if (w instanceof Integer) {
			return Integer.TYPE;
		} else if (w instanceof Long) {
			return Long.TYPE;
		} else if (w instanceof Float) {
			return Float.TYPE;
		} else if (w instanceof Double) {
			return Double.TYPE;
		}
		return null;
	}

	public static Class getPrimitiveFrom(Class klass) {

		String s = klass.getName();

		if (s.equals("java.lang.Boolean")) {
			return Boolean.TYPE;
		} else if (s.equals("java.lang.Byte")) {
			return Byte.TYPE;
		} else if (s.equals("java.lang.Short")) {
			return Short.TYPE;
		} else if (s.equals("java.lang.Character")) {
			return Character.TYPE;
		} else if (s.equals("java.lang.Integer")) {
			return Integer.TYPE;
		} else if (s.equals("java.lang.Long")) {
			return Long.TYPE;
		} else if (s.equals("java.lang.Float")) {
			return Float.TYPE;
		} else if (s.equals("java.lang.Double")) {
			return Double.TYPE;
		}
		return null;
	}

	public static Field getField(Object target, String name) {
		return getField(target.getClass(), name);
	}

	public static Field getField(Class<? extends Object> target, String name) {
		Field fields[] = target.getDeclaredFields();
		for (int i = 0; i < fields.length; i++) {
			if (name.equals(fields[i].getName())) {
				return fields[i];
			}
		}
		return null;
	}

	public static Method findMethodToGet(Class<? extends Object> target,
			String name) {

		StringBuffer sb = new StringBuffer(128);

		sb.append("get").append(name.substring(0, 1).toUpperCase());

		if (name.length() > 1)
			sb.append(name.substring(1));

		try {

			return target.getMethod(sb.toString(), new Class[0]);

		} catch (Exception e) {

		}

		return null;
	}

	public static Method findMethodToInject(Class<? extends Object> target,
			String name, Class source) {

		StringBuffer sb = new StringBuffer(128);

		sb.append("set").append(name.substring(0, 1).toUpperCase());

		if (name.length() > 1)
			sb.append(name.substring(1));

		String methodName = sb.toString();

		Method m = null;

		try {

			m = FindMethod
					.getMethod(target, methodName, new Class[] { source });

		} catch (Exception e) {
		}

		if (m == null) {

			Class primitive = getPrimitiveFrom(source);

			if (primitive != null) {

				try {

					m = target.getMethod(methodName, new Class[] { primitive });

				} catch (Exception e) {
				}

			}
		}

		if (m != null) {
			m.setAccessible(true);
		}

		return m;

	}

	public static Field findFieldToInject(Class<? extends Object> target,
			String name, Class source) {

		Field f = getField(target, name);

		if (f != null) {

			Class<?> type = f.getType();

			if (type.isAssignableFrom(source)) {

				f.setAccessible(true);

				return f;
			}

		}

		return null;
	}

	public static Object getValueToInject(String name, Input input,
			String prefix) {

		if (prefix == null) {

			return input.getValue(name);
		}

		StringBuffer sb = new StringBuffer(64);

		sb.append(prefix).append(PREFIX_SEPARATOR).append(name);

		return input.getValue(sb.toString());

	}

	public static boolean inject(Method m, Object target, Object value,
			Locale loc, boolean tryToConvert) throws Exception {

		Class<?> type = m.getParameterTypes()[0];

		if (type.isAssignableFrom(value.getClass())
				|| checkPrimitives(type, value.getClass())
				|| (tryToConvert && (value = tryToConvert(value, type, loc)) != null)) {

			try {

				m.invoke(target, new Object[] { value });

				return true;

			} catch (Exception e) {

				System.err.println("Error injecting by method: " + value
						+ " in " + target);

				e.printStackTrace();

				throw e;

			}
		}

		return false;

	}

	public static void getObject(Object target, Input input, Locale loc,
			boolean tryField, String prefix, boolean tryToConvert)
			throws Exception {

		Class<? extends Object> targetClass = target.getClass();

		Map<String, Object> setters, fields;

		// see if we have these in cache...

		synchronized (settersMaps) {

			setters = settersMaps.get(targetClass);

			fields = fieldsMaps.get(targetClass);

		}

		// if not in cache, prepare maps for injection...

		if (setters == null) {

			setters = new HashMap<String, Object>();

			fields = null;

			if (tryField) {

				fields = new HashMap<String, Object>();

			}

			prepareForInjection(targetClass, setters, fields);

			synchronized (settersMaps) {

				settersMaps.put(targetClass, setters);

				fieldsMaps.put(targetClass, fields);

			}
		}

		Iterator<String> iter = setters.keySet().iterator();

		while (iter.hasNext()) {

			String var = iter.next();

			Object value = getValueToInject(var, input, prefix);

			if (value == null)
				continue;

			Object obj = setters.get(var);

			// we may have a list of overloaded methods...

			List list = null;

			Method m = null;

			if (obj instanceof List) {

				list = (List) obj;

			} else {

				m = (Method) setters.get(var);

			}

			if (m != null) {

				inject(m, target, value, loc, tryToConvert);

			} else {

				Iterator it = list.iterator();

				while (it.hasNext()) {

					m = (Method) it.next();

					if (inject(m, target, value, loc, tryToConvert))
						break;

				}
			}
		}

		if (fields != null) {

			iter = fields.keySet().iterator();

			while (iter.hasNext()) {

				String var = iter.next();

				Object value = getValueToInject(var, input, prefix);

				if (value == null)
					continue;

				Field f = (Field) fields.get(var);

				Class<?> type = f.getType();

				if (type.isAssignableFrom(value.getClass())
						|| checkPrimitives(type, value.getClass())
						|| (tryToConvert && (value = tryToConvert(value, type,
								loc)) != null)) {

					try {

						f.set(target, value);

					} catch (Exception e) {

						System.err.println("Error injecting by field: " + value
								+ " in " + target);

						e.printStackTrace();

						throw e;

					}
				}
			}
		}
	}

	/**
	 * Extract all properties from bean and place them in output.
	 * 
	 * @param bean
	 *            The beam from where to extract the properties.
	 * @param output
	 *            The output where to place the properties.
	 * @param prefix
	 *            The prefix to use when placing the properties in the output.
	 * @param overwrite
	 *            Overwrite ot not if value is already in the output.
	 */
	public static void setObject(Object bean, Output output, String prefix,
			boolean overwrite) {

		if (bean != null) {

			Method[] methods = bean.getClass().getMethods();

			for (int i = 0; i < methods.length; i++) {

				String name = methods[i].getName();

				if (name.length() > 3 && name.startsWith("get")
						&& !name.equals("getClass")
						&& methods[i].getParameterTypes().length == 0) {

					String adjusted = adjustName(name, prefix);

					if (!overwrite) {

						if (output.getValue(adjusted) != null)
							continue;

					}

					try {
						methods[i].setAccessible(true);
						Object value = methods[i].invoke(bean, new Object[0]);
						output.setValue(adjusted, value);
					} catch (Exception e) {
						System.err
								.println("Error calling method in InjectionUtils: "
										+ name);
						e.printStackTrace();
					}
				}
			}
		}
	}

	/*
	 * This method takes setUsername and returns username.
	 * 
	 * If we have a prefix, then it returns prefix.username.
	 */
	private static String adjustName(String name, String prefix) {
		StringBuffer sb = new StringBuffer(name.length() - 3);
		sb.append(name.substring(3, 4).toLowerCase());
		sb.append(name.substring(4, name.length()));

		if (prefix != null) {

			StringBuffer sb2 = new StringBuffer(128);

			return sb2.append(prefix).append(PREFIX_SEPARATOR).append(
					sb.toString()).toString();

		}

		return sb.toString();
	}

	public static String getProperty(Object bean, String nameProperty)
			throws Exception {
		Class<? extends Object> clazz = bean.getClass();
		Method[] methods = clazz.getMethods();
		for (Method method : methods) {
			String methodName = getter(nameProperty);
			if (method.getName().equals(methodName)
					&& method.getParameterTypes().length == 0)
				return method.invoke(bean, new Object[0]).toString();
		}
		
		Field[] fields = clazz.getDeclaredFields();
		for (Field field : fields) {
			field.setAccessible(true);
			if(field.getName().equals(nameProperty)){
				return field.get(bean).toString();
			}
		}

		return null;
	}

	private static String getter(String name) {
		StringBuilder sb = new StringBuilder(128);

		sb.append("get").append(name.substring(0, 1).toUpperCase());

		if (name.length() > 1)
			sb.append(name.substring(1));
		return sb.toString();
	}

}
